@@ -7,11 +7,14 @@ the PyBpod rig raw data.
 
 Each DatasetType in the IBL pipeline should have one extractor function.
 """
-import ibllib.io.raw_data_loaders as raw
-import numpy as np
-import os
 import logging
+import os
 from pathlib import Path
+
+import cv2
+import numpy as np
+
+import ibllib.io.raw_data_loaders as raw
 from ibllib.misc import version
 
 logger_ = logging.getLogger('ibllib.alf')
@@ -290,17 +293,26 @@ def get_feedback_times_ge5(session_path, data=False):
     # that is grater than the nogo or err trial onset time
     if not data:
         data = raw.load_data(session_path)
-    rw_times = [tr['behavior_data']['States timestamps']['reward'][0][0]
-                for tr in data]
-    sound_times = [tr['behavior_data']['Events timestamps']['BNC2High'] for tr in data]
+    missed_bnc2 = 0
+    rw_times, err_sound_times, merge = [np.zeros([len(data), ]) for _ in range(3)]
 
-    err_sound_times = [x[-1] if len(x) == 2 else np.nan for x in sound_times]
+    for ind, tr in enumerate(data):
+        st = tr['behavior_data']['Events timestamps'].get('BNC2High', None)
+        if not st:
+            st = np.array([np.nan, np.nan])
+            missed_bnc2 += 1
+        # xonar soundcard duplicates events, remove consecutive events too close together
+        st = np.delete(st, np.where(np.diff(st) < 0.020)[0] + 1)
+        rw_times[ind] = tr['behavior_data']['States timestamps']['reward'][0][0]
+        # get the error sound only if the reward is nan
+        err_sound_times[ind] = st[-1] if st.size >= 2 and np.isnan(rw_times[ind]) else np.nan
+    if missed_bnc2 == len(data):
+        logger_.warning('No BNC2 for feedback times, filling error trials NaNs')
+    merge *= np.nan
+    merge[~np.isnan(rw_times)] = rw_times[~np.isnan(rw_times)]
+    merge[~np.isnan(err_sound_times)] = err_sound_times[~np.isnan(err_sound_times)]
 
-    assert sum(np.isnan(rw_times) &
-               np.isnan(err_sound_times)) == 0
-    merge = np.array([np.array(times)[~np.isnan(times)] for times in
-                      zip(rw_times, err_sound_times)]).squeeze()
-    return np.array(merge)
+    return merge
 
 
 def get_feedback_times(session_path, save=False, data=False, settings=False):
@@ -627,8 +639,6 @@ def get_goCueOnset_times(session_path, save=False, data=False, settings=False):
 
     :param session_path: Absolute path of session folder
     :type session_path: str
-    :param save: wether to save the corresponding alf file
-                 to the alf folder, defaults to False
     :param save: bool, optional
     :return: numpy.ndarray
     :rtype: dtype('float64')
@@ -640,23 +650,29 @@ def get_goCueOnset_times(session_path, save=False, data=False, settings=False):
     if settings is None or settings['IBLRIG_VERSION_TAG'] == '':
         settings = {'IBLRIG_VERSION_TAG': '100.0.0'}
 
-    go_cue_times = []
-    for tr in data:
+    go_cue_times = np.zeros([len(data), ])
+    for ind, tr in enumerate(data):
         if get_port_events(tr, 'BNC2'):
-            go_cue_times.append(tr['behavior_data']['Events timestamps']['BNC2High'][0])
+            bnchigh = tr['behavior_data']['Events timestamps'].get('BNC2High', None)
+            if bnchigh:
+                go_cue_times[ind] = bnchigh[0]
+                continue
+            bnclow = tr['behavior_data']['Events timestamps'].get('BNC2Low', None)
+            if bnclow:
+                go_cue_times[ind] = bnclow[0] - 0.1
+                continue
+            go_cue_times[ind] = np.nan
         else:
-            go_cue_times.append(np.nan)
-
-    go_cue_times = np.array(go_cue_times)
+            go_cue_times[ind] = np.nan
 
     nmissing = np.sum(np.isnan(go_cue_times))
     # Check if all stim_syncs have failed to be detected
     if np.all(np.isnan(go_cue_times)):
-        logger_.error(f'{session_path}: Missing ALL BNC1 stimulus ({nmissing} trials')
+        logger_.error(f'{session_path}: Missing ALL BNC2 stimulus ({nmissing} trials')
 
     # Check if any stim_sync has failed be detected for every trial
     if np.any(np.isnan(go_cue_times)):
-        logger_.warning(f'{session_path}: Missing BNC1 stimulus on {nmissing} trials')
+        logger_.warning(f'{session_path}: Missing BNC2 stimulus on {nmissing} trials')
 
     if raw.save_bool(save, '_ibl_trials.goCue_times.npy'):
         check_alf_folder(session_path)
@@ -690,6 +706,95 @@ def get_included_trials_ge5(session_path, data=False, settings=False):
     return trials_included
 
 
+def get_camera_timestamps(session_path, data=False, save=False, settings=False):
+    """
+    Get the camera timestamps from the Bpod
+
+    The camera events are logged only during the events not in between, so the times need
+    to be interpolated
+
+    :param session_path: Absolute path of session folder
+    :param save: bool, optional
+    :return: numpy.ndarray
+    """
+    if not data:
+        data = raw.load_data(session_path)
+    ntrials = len(data)
+
+    cam_times = []
+    n_frames = 0
+    n_out_of_sync = 0
+    for ind in np.arange(ntrials):
+        # get upgoing and downgoing fronts
+        pin = np.array(data[ind]['behavior_data']['Events timestamps'].get('Port1In'))
+        pout = np.array(data[ind]['behavior_data']['Events timestamps'].get('Port1Out'))
+        # some trials at startup may not have the camera working, discard
+        if np.all(pin) is None:
+            continue
+        # if the trial starts in the middle of a square, discard the first downgoing front
+        if pout[0] < pin[0]:
+            pout = pout[1:]
+        # same if the last sample is during an upgoing front, always pout size as it happens last
+        pin = pin[:pout.size]
+        frate = np.median(np.diff(pin))
+        if ind > 0:
+            """
+            assert that the pulses have the same length and that we don't miss frames during
+            the trial, the refresh rate of bpod is 100us
+            """
+            test1 = np.all(np.abs(1 - (pin - pout) / np.median(pin - pout)) < 0.1)
+            test2 = np.all(np.abs(np.diff(pin) - frate) <= 0.00011)
+            if not all([test1, test2]):
+                n_out_of_sync += 1
+        # grow a list of cam times for ech trial
+        cam_times.append(pin)
+        n_frames += pin.size
+
+    if n_out_of_sync > 0:
+        logger_.warning(f"{n_out_of_sync} trials with frame times not within 10% of the"
+                        f" expected sampling rate")
+
+    t_first_frame = np.array([c[0] for c in cam_times])
+    t_last_frame = np.array([c[-1] for c in cam_times])
+    frate = 1 / np.nanmedian(np.array([np.median(np.diff(c)) for c in cam_times]))
+    intertrial_duration = t_first_frame[1:] - t_last_frame[:-1]
+    intertrial_missed_frames = np.int32(np.round(intertrial_duration * frate)) - 1
+
+    # initialize the full times array
+    frame_times = np.zeros(n_frames + int(np.sum(intertrial_missed_frames)))
+    ii = 0
+    for trial, cam_time in enumerate(cam_times):
+        if cam_time is not None:
+            # populate first the recovered times within the trials
+            frame_times[ii: ii + cam_time.size] = cam_time
+            ii += cam_time.size
+        if trial == (len(cam_times) - 1):
+            break
+        # then extrapolate in-between
+        nmiss = intertrial_missed_frames[trial]
+        frame_times[ii: ii + nmiss] = (cam_time[-1] + intertrial_duration[trial] /
+                                       (nmiss + 1) * (np.arange(nmiss) + 1))
+        ii += nmiss
+    # import matplotlib.pyplot as plt
+    # plt.plot(np.diff(frame_times))
+    """
+    if we find a video file, get the number of frames and extrapolate the times using the median
+    frame rate as the video stops after the bpod
+    """
+    video_file = list(session_path.joinpath('raw_video_data').glob('_iblrig_leftCamera*.mp4'))
+    if video_file:
+        cap = cv2.VideoCapture(str(video_file[0]))
+        nframes = cap.get(cv2.CAP_PROP_FRAME_COUNT)
+        if nframes > len(frame_times):
+            to_app = (np.arange(int(nframes - frame_times.size),) + 1) / frate + frame_times[-1]
+            frame_times = np.r_[frame_times, to_app]
+    assert(np.all(np.diff(frame_times) > 0))  # negative diffs implies a big problem
+    if save:
+        fpath = Path(session_path).joinpath('alf', '_ibl_leftCamera.times.npy')
+        np.save(fpath, frame_times)
+    return frame_times
+
+
 def get_included_trials(session_path, save=False, data=False, settings=False):
     if not data:
         data = raw.load_data(session_path)
@@ -747,6 +852,8 @@ def extract_all(session_path, save=False, data=False, settings=False):
             session_path, save=save, data=data, settings=settings)
         go_cue_times = get_goCueOnset_times(
             session_path, save=save, data=data, settings=settings)
+        camera_timestamps = get_camera_timestamps(
+            session_path, save=save, data=data, settings=settings)
         out = {'feedbackType': feedbackType,
                'contrastLeft': contrastLeft,
                'contrastRight': contrastRight,
@@ -762,7 +869,8 @@ def extract_all(session_path, save=False, data=False, settings=False):
                'response_times': response_times,
                'trials_included': trials_included,
                'goCue_times': go_cue_times,
-               'goCueTrigger_times': go_cue_trig_times}
+               'goCueTrigger_times': go_cue_trig_times,
+               'camera_timestamps': camera_timestamps}
     else:
         feedbackType = get_feedbackType(
             session_path, save=save, data=data, settings=settings)
