@@ -1,16 +1,19 @@
 from pathlib import Path, PurePath
 import requests
 import logging
+import os
+import tqdm
 
-from ibllib.misc import is_uuid_string, pprint
+from ibllib.io import hashfile
+from ibllib.misc import pprint
 from ibllib.io.one import OneAbstract
-from alf.io import load_file_content
+from alf.io import load_file_content, remove_uuid_file, is_uuid_string, AlfBunch, get_session_path
 
 import oneibl.webclient as wc
 from oneibl.dataclass import SessionDataInfo
 import oneibl.params
 
-logger_ = logging.getLogger('ibllib')
+_logger = logging.getLogger('ibllib')
 
 
 _ENDPOINTS = {  # keynames are possible input arguments and values are actual endpoints
@@ -135,7 +138,6 @@ class ONE(OneAbstract):
         :param details: returns a second argument with a full dictionary to provide context
         :type details: bool
 
-
         :return: list of strings, plus list of dictionaries if details option selected
         :rtype:  list, list
 
@@ -168,7 +170,7 @@ class ONE(OneAbstract):
 
         # this is a query about datasets: need to unnest session info through the load function
         if keyword == 'dataset-type':
-            dses = self.load(eid, dry_run=True)
+            dses = self.load(eid, dataset_types='__all__', dry_run=True)
             dlist = list(sorted(set(dses.dataset_type)))
             if details:
                 return dses
@@ -176,17 +178,17 @@ class ONE(OneAbstract):
                 return dlist
 
         # get the session information
-        ses = self.alyx.get('/sessions?id=' + eid)
+        ses = self.alyx.rest('sessions', 'read', eid)
 
         if keyword.lower() == 'all':
-            return ses
+            return [ses]
         elif details:
-            return ses[0][keyword], ses
+            return ses[keyword], ses
         else:
-            return ses[0][keyword]
+            return ses[keyword]
 
     def load(self, eid, dataset_types=None, dclass_output=False, dry_run=False, cache_dir=None,
-             download_only=False, clobber=False, offline=False):
+             download_only=False, clobber=False, offline=False, keep_uuid=False):
         """
         From a Session ID and dataset types, queries Alyx database, downloads the data
         from Globus, and loads into numpy array.
@@ -207,6 +209,8 @@ class ONE(OneAbstract):
         :type download_only: bool
         :param clobber: force downloading even if files exists locally
         :type clobber: bool
+        :param keep_uuid: keeps the UUID at the end of the filename (defaults to False)
+        :type keep_uuid: bool
 
         :return: List of numpy arrays matching the size of dataset_types parameter, OR
          a dataclass containing arrays and context data.
@@ -214,9 +218,51 @@ class ONE(OneAbstract):
         """
         # this is a wrapping function to keep signature and docstring accessible for IDE's
         return self._load_recursive(eid, dataset_types=dataset_types, dclass_output=dclass_output,
-                                    dry_run=dry_run, cache_dir=cache_dir,
+                                    dry_run=dry_run, cache_dir=cache_dir, keep_uuid=keep_uuid,
                                     download_only=download_only, clobber=clobber, offline=offline)
 
+    def load_dataset(self, eid, dataset_type, **kwargs):
+        """
+        Load a single dataset from a Session ID and a dataset type.
+
+        :param eid: Experiment ID, for IBL this is the UUID of the Session as per Alyx
+         database. Could be a full Alyx URL:
+         'http://localhost:8000/sessions/698361f6-b7d0-447d-a25d-42afdef7a0da' or only the UUID:
+         '698361f6-b7d0-447d-a25d-42afdef7a0da'.
+        :type eid: str
+        :param dataset_type: Alyx dataset type to be returned.
+        :type dataset_types: str
+
+        :return: A numpy array.
+        :rtype: numpy array
+        """
+        return self._load(eid, dataset_types=[dataset_type], **kwargs)[0]
+
+    def load_object(self, eid, obj, **kwargs):
+        """
+        Load all attributes of an ALF object from a Session ID and an object name.
+
+        :param eid: Experiment ID, for IBL this is the UUID of the Session as per Alyx
+         database. Could be a full Alyx URL:
+         'http://localhost:8000/sessions/698361f6-b7d0-447d-a25d-42afdef7a0da' or only the UUID:
+         '698361f6-b7d0-447d-a25d-42afdef7a0da'.
+        :type eid: str
+        :param obj: Alyx object to load.
+        :type obj: str
+
+        :return: A dictionary-like structure with one key per dataset type for the requested
+         object, and a NumPy array per value.
+        :rtype: AlfBunch instance
+        """
+        dataset_types = [dst for dst in self.list(eid) if dst.startswith(obj)]
+        if len(dataset_types) == 0:
+            _logger.warning(f"{eid} does not contain any {obj} object datasets")
+            return
+        dsets = self._load(eid, dataset_types=dataset_types, **kwargs)
+        return AlfBunch({
+            '.'.join(dataset_types[i].split('.')[1:]): dsets[i]
+            for i in range(len(dataset_types))})
+
     def _load_recursive(self, eid, **kwargs):
         """
         From a Session ID and dataset types, queries Alyx database, downloads the data
@@ -239,7 +285,7 @@ class ONE(OneAbstract):
             return out
 
     def _load(self, eid, dataset_types=None, dclass_output=False, dry_run=False, cache_dir=None,
-              download_only=False, clobber=False, offline=False):
+              download_only=False, clobber=False, offline=False, keep_uuid=False):
         """
         From a Session ID and dataset types, queries Alyx database, downloads the data
         from Globus, and loads into numpy array. Single session only
@@ -259,10 +305,8 @@ class ONE(OneAbstract):
         # a) force the output to be a dictionary that provides context to the data
         # b) download all types that have a data url specified whithin the alf folder
         dataset_types = [dataset_types] if isinstance(dataset_types, str) else dataset_types
-        if not dataset_types:
+        if not dataset_types or dataset_types == ['__all__']:
             dclass_output = True
-            dataset_types = [d['dataset_type'] for d in ses['data_dataset_session_related']
-                             if d['data_url'] and 'alf' in Path(d['data_url']).parts]
         dc = SessionDataInfo.from_session_details(ses, dataset_types=dataset_types, eid=eid_str)
         # loop over each dataset and download if necessary
         for ind in range(len(dc)):
@@ -270,8 +314,9 @@ class ONE(OneAbstract):
                 relpath = PurePath(dc.url[ind].replace(self._par.HTTP_DATA_SERVER, '.')).parents[0]
                 cache_dir_file = PurePath(cache_dir, relpath)
                 Path(cache_dir_file).mkdir(parents=True, exist_ok=True)
-                dc.local_path[ind] = self._download_file(dc.url[ind], str(cache_dir_file),
-                                                         clobber=clobber, offline=offline)
+                dc.local_path[ind] = self._download_file(
+                    dc.url[ind], str(cache_dir_file), clobber=clobber, offline=offline,
+                    keep_uuid=keep_uuid, file_size=dc.file_size[ind], hash=dc.hash[ind])
         # load the files content in variables if requested
         if not download_only:
             for ind, fil in enumerate(dc.local_path):
@@ -283,12 +328,15 @@ class ONE(OneAbstract):
         list_out = []
         for dt in dataset_types:
             if dt not in dc.dataset_type:
-                logger_.warning('dataset ' + dt + ' not found for session: ' + eid_str)
+                _logger.warning('dataset ' + dt + ' not found for session: ' + eid_str)
                 list_out.append(None)
                 continue
             for i, x, in enumerate(dc.dataset_type):
                 if dt == x:
-                    list_out.append(dc.data[i])
+                    if dc.data[i] is not None:
+                        list_out.append(dc.data[i])
+                    else:
+                        list_out.append(dc.local_path[i])
         return list_out
 
     def _ls(self, table=None, verbose=False):
@@ -335,22 +383,43 @@ class ONE(OneAbstract):
 
         >>> one.search_terms()
 
-        :param details: default False, returns also the session details as per the REST response
-        :type details: bool
-        :param limit: default None, limits results (if pagination enabled on server)
-        :type limit: int List of possible search terms
         :param dataset_types: list of dataset_types
         :type dataset_types: list of str
-        :param users: a list of users
-        :type users: list or str
-        :param subjects: a list of subjects nickname
-        :type subjects: list or str
-        :param lab: a str or list of lab names
-        :type lab: list or str
+
         :param date_range: list of 2 strings or list of 2 dates that define the range
         :type date_range: list
-        :param number: session number
+
+        :param details: default False, returns also the session details as per the REST response
+        :type details: bool
+
+        :param lab: a str or list of lab names
+        :type lab: list or str
+
+        :param limit: default None, limits results (if pagination enabled on server)
+        :type limit: int List of possible search terms
+
+        :param location: a str or list of lab location (as per Alyx definition) name
+                         Note: this corresponds to the specific rig, not the lab geographical
+                         location per se
+        :type location: str
+
+        :param number: number of session to be returned; will take the first n sessions found
         :type number: str or int
+
+        :param performance_lte / performance_gte: search only for sessions whose performance is
+        less equal or greater equal than a pre-defined threshold as a percentage (0-100)
+        :type performance_gte: float
+
+        :param subjects: a list of subjects nickname
+        :type subjects: list or str
+
+        :param task_protocol: a str or list of task protocol name (can be partial, i.e.
+                              any task protocol containing that str will be found)
+        :type task_protocol: str
+
+        :param users: a list of users
+        :type users: list or str
+
         :return: list of eids, if details is True, also returns a list of json dictionaries,
          each entry corresponding to a matching session
         :rtype: list, list
@@ -370,7 +439,7 @@ class ONE(OneAbstract):
         for k in kwargs.keys():
             # check that the input matches one of the defined filters
             if k not in SEARCH_TERMS:
-                logger_.error(f'"{k}" is not a valid search keyword' + '\n' +
+                _logger.error(f'"{k}" is not a valid search keyword' + '\n' +
                               "Valid keywords are: " + str(set(SEARCH_TERMS.values())))
                 return
             # then make sure the field is formatted properly
@@ -386,9 +455,19 @@ class ONE(OneAbstract):
             url += f'&limit={limit}'
         # implements the loading itself
         ses = self.alyx.get(url)
-        eids = [s['url'] for s in ses]  # flattens session info
+        if len(ses) > 2500:
+            eids = [s['url'] for s in tqdm.tqdm(ses)]  # flattens session info
+        else:
+            eids = [s['url'] for s in ses]
         eids = [e.split('/')[-1] for e in eids]  # remove url to make it portable
         if details:
+            for s in ses:
+                if all([s.get('lab'), s.get('subject'), s.get('start_time')]):
+                    s['local_path'] = str(Path(self._par.CACHE_DIR, s['lab'], 'Subjects',
+                                               s['subject'], s['start_time'][:10],
+                                               str(s['number']).zfill(3)))
+                else:
+                    s['local_path'] = None
             return eids, ses
         else:
             return eids
@@ -401,14 +480,32 @@ class ONE(OneAbstract):
             cache_dir = str(PurePath(Path.home(), "Downloads", "FlatIron"))
         return cache_dir
 
-    def _download_file(self, url, cache_dir, clobber=False, offline=False):
-        local_path = wc.http_download_file(url,
-                                           username=self._par.HTTP_DATA_SERVER_LOGIN,
-                                           password=self._par.HTTP_DATA_SERVER_PWD,
-                                           cache_dir=str(cache_dir),
-                                           clobber=clobber,
-                                           offline=offline)
-        return local_path
+    def _download_file(self, url, cache_dir, clobber=False, offline=False, keep_uuid=False,
+                       file_size=None, hash=None):
+        local_path = cache_dir + os.sep + os.path.basename(url)
+        if not keep_uuid:
+            local_path = remove_uuid_file(local_path, dry=True)
+        if Path(local_path).exists():
+            # overwrites the file if the expected filesize is different from the cached filesize
+            if file_size and Path(local_path).stat().st_size != file_size:
+                clobber = True
+            # overwrites the file if the expected hash is different from the cached hash
+            if hash and hashfile.md5(Path(local_path)) != hash:
+                clobber = True
+        # if there is no cached file, download
+        else:
+            clobber = True
+        if clobber:
+            local_path = wc.http_download_file(url,
+                                               username=self._par.HTTP_DATA_SERVER_LOGIN,
+                                               password=self._par.HTTP_DATA_SERVER_PWD,
+                                               cache_dir=str(cache_dir),
+                                               clobber=clobber,
+                                               offline=offline)
+        if keep_uuid:
+            return local_path
+        else:
+            return remove_uuid_file(local_path)
 
     @staticmethod
     def search_terms():
@@ -437,6 +534,51 @@ class ONE(OneAbstract):
         """
         oneibl.params.setup()
 
+    def path_from_eid(self, eid: str, grep_str=None) -> Path:
+        # If eid is a list of eIDs recurse through list and return the results
+        if isinstance(eid, list):
+            path_list = []
+            for p in eid:
+                path_list.append(self.path_from_eid(p, grep_str=grep_str))
+            return path_list
+        # If not valid return None
+        if not is_uuid_string(eid):
+            print(eid, " is not a valid eID/UUID string")
+            return
+        # Load data, if no data present on disk return None
+        data = self._load(eid, download_only=True, offline=True)
+        if not data.local_path:
+            return None
+        # If user defined a grep list of specific files return paths to files
+        if grep_str is not None:
+            files = [x for x in data.local_path if grep_str in str(x)]
+            return files
+        # If none of the above happen return the session path of the first file you find
+        session_path = get_session_path(data.local_path[0])
+
+        return session_path
+
+    def eid_from_path(self, path_obj):
+        # If path_obj is a list recurse through it and return a list
+        if isinstance(path_obj, list):
+            path_obj = [Path(x) for x in path_obj]
+            eid_list = []
+            for p in path_obj:
+                eid_list.append(self.eid_from_path(p))
+            return eid_list
+        # else ensure the path ends with mouse,date, number
+        path_obj = Path(path_obj)
+        session_path = get_session_path(path_obj)
+        # if path does not have a date and a number return None
+        if session_path is None:
+            return None
+        # search for subj, date, number XXX: hits the DB
+        uuid = self.search(subjects=session_path.parts[-3],
+                           date_range=session_path.parts[-2],
+                           number=session_path.parts[-1])
+        # Return the uuid if any
+        return uuid[0] if uuid else None
+
 
 def _validate_date_range(date_range):
     """
